import React, { Component } from 'react';
import { StyleProp, ViewStyle } from 'react-native';
import MapView from 'react-native-maps';

/**
 * @type {string}
 */
declare const DRIVING = "DRIVING";
/**
 * @type {string}
 */
declare const WALKING = "WALKING";
/**
 * @type {string}
 */
declare const TRANSIT = "TRANSIT";
/**
 * @type {string}
 */
declare const BICYCLING = "BICYCLING";
/**
 * @default export
 */
declare const _default$2: {
    DRIVING: string;
    WALKING: string;
    TRANSIT: string;
    BICYCLING: string;
};

/**
 * @imports
 */

type TravelMode$1 = typeof DRIVING | typeof WALKING | typeof TRANSIT | typeof BICYCLING;
interface TravelModeBoxProps {
    backgroundColor?: string;
    borderColor?: string;
    borderWidth?: number;
    borderRadius?: number;
    contentPadding?: number;
    inverseTextColor?: string;
    modes?: TravelMode$1[];
    selected?: TravelMode$1;
    defaultValue?: TravelMode$1;
    style?: StyleProp<ViewStyle>;
    onChange?: (mode: TravelMode$1) => void;
    theme?: string;
    invertKeyLabel?: boolean;
    fontFamily?: string;
    fontSize?: number;
    useIcons?: boolean;
}
/**
 * @component
 */
declare class TravelModeBox extends Component<TravelModeBoxProps> {
    /**
     * defaultProps
     * @type {}
     */
    static defaultProps: {
        backgroundColor: string;
        borderColor: string;
        borderWidth: number;
        borderRadius: number;
        contentPadding: number;
        inverseTextColor: string;
        defaultValue: string;
        selected: any;
        style: {};
        onChange: any;
        theme: any;
        invertKeyLabel: boolean;
        fontSize: number;
        fontFamily: any;
        useIcons: boolean;
        modes: string[];
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: TravelModeBoxProps);
    render(): React.JSX.Element;
}

/**
 * @imports
 */

type TravelMode = typeof DRIVING | typeof WALKING | typeof TRANSIT | typeof BICYCLING;
/**
 * Component props interface
 */
interface TravelModeLabelProps {
    size?: number;
    opacity?: number;
    color?: string;
    fontFamily?: string;
    fontSize?: number;
    useIcon?: boolean;
    useLabel?: boolean;
    mode?: TravelMode;
}
/**
 * @component
 */
declare class TravelModeLabel extends Component<TravelModeLabelProps> {
    /**
     * defaultProps
     * @type {}
     */
    static defaultProps: {
        color: any;
        opacity: number;
        fontSize: number;
        fontFamily: any;
        useIcon: boolean;
        useLabel: boolean;
        mode: any;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: TravelModeLabelProps);
    /**
     * render
     * @returns {JSX.Element | null}
     */
    render(): React.JSX.Element;
}

/**
 * @imports
 */

/**
 * DirectionsListView props interface
 */
interface DirectionsListViewProps {
    route: Route;
    fontFamily?: string;
    fontFamilyBold?: string;
    showOriginDestinationHeader?: boolean;
    displayTravelMode?: boolean;
}
/**
 * @component
 */
declare class DirectionsListView extends Component<DirectionsListViewProps> {
    /**
     * defaultProps
     */
    static defaultProps: {
        fontFamily: any;
        fontFamilyBold: any;
        showOriginDestinationHeader: boolean;
        displayTravelMode: boolean;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: DirectionsListViewProps);
    /**
     * render
     * @returns {JSX.Element | null}
     */
    render(): React.JSX.Element;
}

/**
 * @enum {string}
 */
declare enum MarkerTypes {
    ORIGIN = "ORIGIN",
    DESTINATION = "DESTINATION",
    POSITION_DOT = "POSITION_DOT",
    POSITION_ARROW = "POSITION_ARROW"
}
type MarkerType = (typeof MarkerTypes)[keyof typeof MarkerTypes];

/**
 * @enum {string}
 */
declare enum PolylineTypes {
    ROUTE = "ROUTE",
    ROUTE_ALTERNATIVE = "ROUTE_ALTERNATIVE",
    PENDING = "PENDING",
    NEXT = "NEXT"
}
type PolylineType = (typeof PolylineTypes)[keyof typeof PolylineTypes];

/**
 * @import
 */

/**
 * Theme interfaces
 */
interface ThemeMarker {
    icon: string;
    color: string;
    fontSize?: number;
    size?: number;
    backgroundColor?: string;
}
interface ThemePolyline {
    fillColor: string;
    strokeColor: string;
    strokeWidth: number;
    borderWidth: number;
}
interface ThemeSettings {
    Markers: {
        [key in MarkerType]: ThemeMarker;
    };
    Polylines: {
        [key in PolylineType]: ThemePolyline;
    };
}

/**
 * @imports
 */

interface RouteMarkerProps {
    coordinate: {
        latitude: number;
        longitude: number;
    };
    type: MarkerType;
    theme: Partial<ThemeSettings>;
}

/**
 * @imports
 */

/**
 * Interface for coordinate data
 */
interface Coordinate {
    latitude: number;
    longitude: number;
}
/**
 * Interface for component state
 */
interface MapViewNavigationState {
    route: Route | false;
    markers: RouteMarkerProps[];
    position: Position;
    navigationMode: string;
    travelMode: string;
    stepIndex: number | false;
    step?: Step | false;
}
/**
 * Interface for component props
 */
interface MapViewNavigationProps {
    /**
     * Origin point for directions
     */
    origin?: string | Coordinate | false;
    /**
     * Destination point for directions
     */
    destination?: string | Coordinate | false;
    /**
     * Google Maps API key
     */
    apiKey: string;
    /**
     * Language for directions and geocoding
     */
    language?: string;
    /**
     * Reference to the map component
     */
    map?: () => MapView;
    /**
     * Navigation mode (IDLE, ROUTE, NAVIGATION)
     */
    navigationMode?: string;
    /**
     * Travel mode (DRIVING, WALKING, BICYCLING, TRANSIT)
     */
    travelMode?: string;
    /**
     * Maximum zoom level
     */
    maxZoom?: number;
    /**
     * Minimum zoom level
     */
    minZoom?: number;
    /**
     * Duration for animations in milliseconds
     */
    animationDuration?: number;
    /**
     * Viewing angle during navigation
     */
    navigationViewingAngle?: number;
    /**
     * Zoom level during navigation
     */
    navigationZoomLevel?: number;
    /**
     * Zoom quantifier for direction view
     */
    directionZoomQuantifier?: number;
    /**
     * Callback when route changes
     */
    onRouteChange?: (route: Route) => void;
    /**
     * Callback when step changes
     */
    onStepChange?: (step: Step | false, nextStep?: Step) => void;
    /**
     * Callback when navigation starts
     */
    onNavigationStarted?: () => void;
    /**
     * Callback when navigation completes
     */
    onNavigationCompleted?: () => void;
    /**
     * Distance to trigger step change
     */
    routeStepDistance?: number;
    /**
     * Inner radius tolerance for step
     */
    routeStepInnerTolerance?: number;
    /**
     * Center radius tolerance for step
     */
    routeStepCenterTolerance?: number;
    /**
     * Course tolerance in degrees
     */
    routeStepCourseTolerance?: number;
    /**
     * Whether to show debug markers
     */
    displayDebugMarkers?: boolean;
    /**
     * Whether to simulate navigation
     */
    simulate?: boolean;
    /**
     * Additional options for directions
     */
    options?: object;
    /**
     * Theme for styling
     */
    theme?: ThemeSettings;
}
/**
 * MapViewNavigation component for handling routing and navigation on maps
 */
declare class MapViewNavigation extends Component<MapViewNavigationProps, MapViewNavigationState> {
    /**
     * Default props for the component
     */
    static defaultProps: Partial<MapViewNavigationProps>;
    private geoCoder;
    private directionsCoder;
    private traps;
    private theme;
    private aspectRatio;
    private watchId;
    private simulator?;
    private stepIndex;
    /**
     * @constructor
     * @param props Component props
     */
    constructor(props: MapViewNavigationProps);
    /**
     * @componentDidMount
     */
    componentDidMount(): void;
    /**
     * @componentWillUnmount
     */
    componentWillUnmount(): void;
    /**
     * @componentDidUpdate
     * @param prevProps Previous props
     */
    componentDidUpdate(prevProps: MapViewNavigationProps): void;
    /**
     * getCoordinates
     * @param address Address to geocode
     * @returns {Promise<Array>} Coordinates
     */
    getCoordinates(address: string): Promise<MinimizedResult | MinimizedResult[]>;
    /**
     * getCoordinatesRaw
     * @param address Address to geocode
     * @returns {Promise<Array>} Raw coordinates
     */
    getCoordinatesRaw(address: string): Promise<LocationResult | LocationResult[]>;
    /**
     * getZoomValue
     * @param level Zoom level
     * @returns Region delta values
     */
    getZoomValue(level: number): {
        latitudeDelta: number;
        longitudeDelta: number;
    };
    /**
     * getBoundingBoxZoomValue
     * @param b Bounding box coordinates
     * @param quantifier Zoom quantifier
     * @returns Region delta values
     */
    getBoundingBoxZoomValue(b: Coordinate[], quantifier?: number): {
        latitudeDelta: number;
        longitudeDelta: number;
    } | null;
    /**
     * updatePosition
     * @param coordinate Coordinate to update to
     * @param duration Animation duration
     */
    updatePosition(coordinate: Coordinate, duration?: number): void;
    /**
     * updateBearing
     * @param bearing New bearing
     * @param duration Animation duration
     */
    updateBearing(bearing: number, duration?: number): void;
    /**
     * updateStep
     * @param stepIndex Index of the step to update to
     */
    updateStep(stepIndex?: number): void;
    /**
     * setPosition
     * @param position Position to set
     */
    setPosition(position: Position): void;
    /**
     * clearRoute
     * @void
     */
    clearRoute(): void;
    /**
     * updateRoute
     * @param origin Origin point
     * @param destination Destination point
     * @param navigationMode Navigation mode
     * @param options Route options
     */
    updateRoute(origin?: string | Coordinate | false, destination?: string | Coordinate | false, navigationMode?: string | false, options?: object | null): void;
    /**
     * Prepares the route
     * @param origin Origin point
     * @param destination Destination point
     * @param options Route options
     * @param testForRoute Whether to test for existing route
     * @returns Promise resolving to route
     */
    prepareRoute(origin: string | Coordinate, destination: string | Coordinate, options?: object | false, testForRoute?: boolean): Promise<Route>;
    /**
     * displayRoute
     * @param origin Origin point
     * @param destination Destination point
     * @param options Route options
     * @returns Promise resolving to route
     */
    displayRoute(origin: string | Coordinate, destination: string | Coordinate, options?: object | false): Promise<Route>;
    /**
     * navigateRoute
     * @param origin Origin point
     * @param destination Destination point
     * @param options Route options
     * @returns Promise resolving to route
     */
    navigateRoute(origin: string | Coordinate, destination: string | Coordinate, options?: object | false): Promise<Route>;
    /**
     * getRouteMarkers
     * @param route Route data
     * @returns Route markers as JSX
     */
    getRouteMarkers(route: Route | false): React.ReactElement[] | null;
    /**
     * getPositionMarker
     * @param position Position data
     * @param navigationMode Current navigation mode
     * @returns Position marker as JSX
     */
    getPositionMarker(position: Position, navigationMode: string): React.ReactElement | null;
    /**
     * Route Polycons
     * @param route Route data
     * @returns Route polylines as JSX
     */
    getRoutePolylines(route: Route | false): (React.ReactElement | null)[] | null;
    /**
     * getDebugShapes
     * @param route Route data
     * @returns Debug shapes as JSX
     */
    getDebugShapes(route: Route | false): React.ReactElement[];
    /**
     * @render
     * @returns {*}
     */
    render(): React.ReactElement;
}

/**
 * @imports
 */

/**
 * @component
 */
declare class ManeuverArrow extends Component<ManeuverArrowProps> {
    /**
     * defaultProps
     */
    static defaultProps: {
        maneuver: any;
        size: number;
        opacity: number;
        color: string;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: ManeuverArrowProps);
    /**
     * render
     * @returns {XML}
     */
    render(): React.JSX.Element;
}

interface ManeuverLabelProps$1 {
    fontFamilyBold?: string;
    fontFamily?: string;
    fontSize?: number;
    fontColor?: string;
    color?: string;
}
/**
 * @styles
 */
declare const styles: (props: ManeuverLabelProps$1) => {
    bold: {
        color: string;
        flexWrap: "wrap";
        fontFamily: string;
        fontSize: number;
        fontWeight: "bold";
    };
    durationDistance: {
        color: string;
        flexWrap: "wrap";
        fontFamily: string;
        fontSize: number;
        opacity: number;
    };
    extra: {
        color: string;
        flexWrap: "wrap";
        fontFamily: string;
        fontSize: number;
        marginTop: number;
    };
    /**
     * @maneuverLabel
     */
    maneuverLabel: {
        flexDirection: "row";
    };
    /**
     * @fonts
     */
    regular: {
        color: string;
        flexWrap: "wrap";
        fontFamily: string;
        fontSize: number;
    };
};
type ManeuverLabelStyles = ReturnType<typeof styles>;

/**
 * @imports
 */

/**
 * @component
 */
declare class ManeuverLabel extends Component<ManeuverLabelProps> {
    /**
     * defaultProps
     */
    static defaultProps: {
        instructions: string;
        fontFamily: any;
        fontFamilyBold: any;
        fontSize: number;
        fontColor: any;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: ManeuverLabelProps);
    /**
     * getParsedInstructions
     * @param styles
     * @returns {*}
     */
    getParsedInstructions(styles: ManeuverLabelStyles): React.JSX.Element;
    /**
     * render
     * @returns {XML}
     */
    render(): React.JSX.Element;
}

/**
 * @imports
 */

/**
 * @component
 */
declare class ManeuverView extends Component<ManeuverViewProps> {
    /**
     * defaultProps
     */
    static defaultProps: {
        step: any;
        fontFamily: any;
        fontFamilyBold: any;
        backgroundColor: string;
        fontSize: number;
        arrowSize: number;
        arrowColor: string;
        withCloseButton: boolean;
        onClose: any;
        onPress: any;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: ManeuverViewProps);
    /**
     * render
     * @returns {JSX.Element | null}
     */
    render(): React.JSX.Element;
}

/**
 * @imports
 */

/**
 * @component
 */
declare class DurationDistanceLabel extends Component<DurationDistanceLabelProps> {
    /**
     * Default props for the component
     */
    static defaultProps: Partial<DurationDistanceLabelProps>;
    /**
     * @constructor
     * @param props Component props
     */
    constructor(props: DurationDistanceLabelProps);
    /**
     * render
     * @returns React element
     */
    render(): React.JSX.Element;
}

/**
 * @imports
 */

/**
 * @component
 */
declare class DurationDistanceView extends Component<DurationDistanceViewProps> {
    /**
     * defaultProps
     */
    static defaultProps: {
        step: any;
        fontFamily: any;
        fontFamilyBold: any;
        fontSize: number;
        arrowSize: number;
        arrowColor: string;
        withCloseButton: boolean;
        onClose: any;
        onPress: any;
    };
    /**
     * @constructor
     * @param props
     */
    constructor(props: DurationDistanceViewProps);
    /**
     * render
     * @returns {XML}
     */
    render(): React.JSX.Element;
}

/**
 * Arrows
 * @type {{}}
 */
declare const _default$1: {
    /**
     * @common
     */
    watch: string;
    timer3: string;
    timer10: string;
    timerAv: string;
    timer: string;
    menu: string;
    zoomOutMap: string;
    zoomOut: string;
    zoomIn: string;
    linearScale: string;
    room: string;
    place: string;
    zoomOutMap2: string;
    locationOff: string;
    locationCity: string;
    locationDisabled: string;
    locationSearching: string;
    myLocation: string;
    editLocation: string;
    addLocation: string;
    map: string;
    directionsTransit: string;
    directionsWalk: string;
    directionsBike: string;
    cardTravel: string;
    home: string;
    directionsDriving: string;
    navigate: string;
    target: string;
    location2: string;
    location3: string;
    compassDirection: string;
    compassDot: string;
    /**
     * @arrows
     * turn-slight-left, turn-sharp-left, uturn-left, turn-left, turn-slight-right, turn-sharp-right,
     * uturn-right, turn-right, straight, ramp-left, ramp-right, merge, fork-left, fork-right, ferry,
     * ferry-train, roundabout-left, roundabout-right
     */
    arriveLeft: string;
    arriveRight: string;
    arriveStraight: string;
    arrive: string;
    close: string;
    continueLeft: string;
    continueRight: string;
    continueSlightLeft: string;
    continueSlightRight: string;
    continueStraight: string;
    continueUturn: string;
    continue: string;
    keepLeft: string;
    keepRight: string;
    departLeft: string;
    departRight: string;
    departStraight: string;
    depart: string;
    endOfRoadLeft: string;
    endOfRoadRight: string;
    flag: string;
    forkLeft: string;
    forkRight: string;
    forkSlightLeft: string;
    forkSlightRight: string;
    forkStraight: string;
    fork: string;
    invalidLeft: string;
    invalidRight: string;
    invalidSlightLeft: string;
    invalidSlightRight: string;
    invalidStraight: string;
    invalidUturn: string;
    invalid: string;
    straight: string;
    merge: string;
    mergeLeft: string;
    mergeRight: string;
    mergeSlightLeft: string;
    mergeSlightRight: string;
    mergeStraight: string;
    newNameLeft: string;
    newNameRight: string;
    newNameSharpLeft: string;
    newNameSharpRight: string;
    newNameSlightLeft: string;
    newNameSlightRight: string;
    newNameStraight: string;
    notificaitonSharpRight: string;
    notificationLeft: string;
    notificationRight: string;
    notificationSharpLeft: string;
    notificationSlightLeft: string;
    notificationSlightRight: string;
    notificationStraight: string;
    offRampLeft: string;
    offRampRight: string;
    offRampSlightLeft: string;
    offRampSlightRight: string;
    rampLeft: string;
    rampRight: string;
    rampSlightLeft: string;
    rampSlightRight: string;
    onRampLeft: string;
    onRampRight: string;
    onRampSharpLeft: string;
    onRampSharpRight: string;
    onRampSlightLeft: string;
    onRampSlightRight: string;
    onRampStraight: string;
    rotaryLeft: string;
    rotaryRight: string;
    rotarySharpLeft: string;
    rotarySharpRight: string;
    rotarySlightLeft: string;
    rotarySlightRight: string;
    rotaryStraight: string;
    rotary: string;
    roundaboutLeft: string;
    roundaboutRight: string;
    roundaboutSharpLeft: string;
    roundaboutSharpRight: string;
    roundaboutSlightLeft: string;
    roundaboutSlightRight: string;
    roundaboutStraight: string;
    roundabout: string;
    turnLeft: string;
    turnRight: string;
    turnSharpLeft: string;
    turnSharpRight: string;
    turnSlightLeft: string;
    turnSlightRight: string;
    turnStraight: string;
    updown: string;
};

/**
 * @export
 */
declare const _default: {
    IDLE: string;
    ROUTE: string;
    NAVIGATION: string;
};

/**
 * @file Geocoder.ts
 * @description A TypeScript wrapper for the Google Maps Geocoding API that provides methods to convert
 * between addresses and geographic coordinates. It includes functionality to format and parse
 * Google's geocoding responses into a more manageable structure.
 *
 * @see {@link https://developers.google.com/maps/documentation/geocoding/overview|Google Geocoding API Documentation}
 */
/**
 * A wrapper class for Google's Geocoding API that provides methods to convert
 * between addresses and geographic coordinates
 *
 * @class Geocoder
 * @example
 * // Create a new geocoder instance
 * const geocoder = new Geocoder('YOUR_GOOGLE_API_KEY', { language: 'en' });
 *
 * // Get location information from an address
 * geocoder.getFromLocation('1600 Amphitheatre Parkway, Mountain View, CA')
 *   .then(results => {
 *     console.log(results);
 *     // Use minimizeResults to get a simplified version
 *     const minimized = geocoder.minimizeResults(results);
 *     console.log(minimized);
 *   })
 *   .catch(error => console.error(error));
 */
declare class Geocoder {
    private apiKey;
    private options;
    /**
     * Mapping between Google's address component types and our simplified naming schema
     *
     * @static
     * @type {AddressComponentMappingType}
     * @example
     * {
     *   street_number: 'number',  // Maps Google's 'street_number' to our 'number'
     *   route: 'street',          // Maps Google's 'route' to our 'street'
     *   // ...and so on
     * }
     */
    static AddressComponentMapping: AddressComponentMappingType;
    /**
     * Creates a new Geocoder instance
     *
     * @constructor
     * @param {string} apiKey - Google Maps API key
     * @param {GeocoderOptions|false} [options=false] - Additional options for the geocoder
     * @throws {Error} Will throw an error if apiKey is not provided
     */
    constructor(apiKey: string, options?: GeocoderOptions | false);
    /**
     * Geocodes an address or reverse geocodes coordinates
     *
     * @async
     * @param {...any[]} params - Either a string address or an object with query parameters
     * @returns {Promise<LocationResult[]>} A promise that resolves to an array of location results
     * @throws {Error} Will reject if the API returns a non-OK status or if there's a network error
     *
     * @example
     * // With a string address
     * geocoder.getFromLocation('1600 Amphitheatre Parkway, Mountain View, CA')
     *   .then(results => console.log(results));
     *
     * // With query parameters
     * geocoder.getFromLocation({
     *   address: '1600 Amphitheatre Parkway',
     *   components: 'country:US'
     * })
     *   .then(results => console.log(results));
     */
    getFromLocation(...params: any[]): Promise<LocationResult[]>;
    /**
     * Reverse geocodes coordinates to get address information
     *
     * @param {number} lat - Latitude in decimal degrees
     * @param {number} lng - Longitude in decimal degrees
     * @returns {Promise<LocationResult[]>} A promise that resolves to an array of location results
     * @throws {Error} Will reject if the API returns a non-OK status or if there's a network error
     *
     * @example
     * geocoder.getFromLatLng(37.4224764, -122.0842499)
     *   .then(results => console.log(results));
     */
    getFromLatLng(lat: number, lng: number): Promise<LocationResult[]>;
    /**
     * Converts Google's geocoding results into a simplified format
     *
     * @param {LocationResult[]} results - The results from getFromLocation or getFromLatLng
     * @returns {MinimizedResult[]} An array of simplified location results
     *
     * @example
     * geocoder.getFromLocation('1600 Amphitheatre Parkway, Mountain View, CA')
     *   .then(results => {
     *     const minimized = geocoder.minimizeResults(results);
     *     console.log(minimized);
     *   });
     */
    minimizeResults(results: LocationResult[]): MinimizedResult[];
    /**
     * Simplifies Google's address components into a more manageable structure
     *
     * @param {AddressComponent[]} components - The address components from a geocoding result
     * @returns {MinimizedAddressComponents} A simplified representation of the address components
     *
     * @example
     * // Result example:
     * {
     *   street: { short: "5th Ave", long: "Fifth Avenue" },
     *   city: { short: "NY", long: "New York" },
     *   state: { short: "NY", long: "New York" },
     *   country: { short: "US", long: "United States" }
     * }
     */
    minimizeAddressComponents(components: AddressComponent[]): MinimizedAddressComponents;
}

/**
 * @imports
 */

export { DirectionsListView, DurationDistanceLabel, DurationDistanceView, Geocoder, ManeuverArrow, ManeuverLabel, ManeuverView, _default as NavigationModes, _default$1 as TravelIcons, TravelModeBox, TravelModeLabel, _default$2 as TravelModes, MapViewNavigation as default };
