import { Component, createElement } from 'react';
import OptionGroupBox from 'react-native-optiongroup';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { StyleSheet, View, Text, ScrollView, Dimensions, TouchableOpacity } from 'react-native';
import { Marker, Circle, Polyline } from 'react-native-maps';
import geolocation from '@react-native-community/geolocation';
import * as GeoLib2 from 'geolib';
import { logger, consoleTransport } from 'react-native-logs';

// src/components/TravelModeBox/index.tsx

// src/constants/NavigationIcons.ts
var NavigationIconsFont = {
  fontFamily: "Navigation"
};
var NavigationIcons_default = {
  /**
   * @common
   */
  watch: "\uE957",
  timer3: "\uE958",
  timer10: "\uE959",
  timerAv: "\uE95A",
  timer: "\uE95B",
  menu: "\uE95C",
  zoomOutMap: "\uE95D",
  zoomOut: "\uE95E",
  zoomIn: "\uE95F",
  linearScale: "\uE960",
  room: "\uE961",
  place: "\uE961",
  zoomOutMap2: "\uE962",
  locationOff: "\uE963",
  locationCity: "\uE964",
  locationDisabled: "\uE965",
  locationSearching: "\uE966",
  myLocation: "\uE967",
  editLocation: "\uE968",
  addLocation: "\uE969",
  map: "\uE96A",
  directionsTransit: "\uE96B",
  directionsWalk: "\uE96C",
  directionsBike: "\uE96D",
  cardTravel: "\uE96E",
  home: "\uE96F",
  directionsDriving: "\uE970",
  navigate: "\uE975",
  target: "\uE971",
  location2: "\uE972",
  location3: "\uE973",
  compassDirection: "\uE974",
  compassDot: "\uE977",
  /**
   * @arrows
   * turn-slight-left, turn-sharp-left, uturn-left, turn-left, turn-slight-right, turn-sharp-right,
   * uturn-right, turn-right, straight, ramp-left, ramp-right, merge, fork-left, fork-right, ferry,
   * ferry-train, roundabout-left, roundabout-right
   */
  arriveLeft: "\uE900",
  arriveRight: "\uE901",
  arriveStraight: "\uE902",
  arrive: "\uE903",
  close: "\uE904",
  continueLeft: "\uE905",
  continueRight: "\uE906",
  continueSlightLeft: "\uE907",
  continueSlightRight: "\uE908",
  continueStraight: "\uE909",
  continueUturn: "\uE90A",
  continue: "\uE90B",
  keepLeft: "\uE907",
  keepRight: "\uE908",
  departLeft: "\uE90C",
  departRight: "\uE90D",
  departStraight: "\uE90E",
  depart: "\uE90F",
  endOfRoadLeft: "\uE910",
  endOfRoadRight: "\uE911",
  flag: "\uE912",
  forkLeft: "\uE913",
  forkRight: "\uE914",
  forkSlightLeft: "\uE915",
  forkSlightRight: "\uE916",
  forkStraight: "\uE917",
  fork: "\uE918",
  invalidLeft: "\uE919",
  invalidRight: "\uE91A",
  invalidSlightLeft: "\uE91B",
  invalidSlightRight: "\uE91C",
  invalidStraight: "\uE91D",
  invalidUturn: "\uE91E",
  invalid: "\uE91F",
  straight: "\uE91F",
  merge: "\uE976",
  mergeLeft: "\uE920",
  mergeRight: "\uE921",
  mergeSlightLeft: "\uE922",
  mergeSlightRight: "\uE923",
  mergeStraight: "\uE924",
  newNameLeft: "\uE925",
  newNameRight: "\uE926",
  newNameSharpLeft: "\uE927",
  newNameSharpRight: "\uE928",
  newNameSlightLeft: "\uE929",
  newNameSlightRight: "\uE92A",
  newNameStraight: "\uE92B",
  notificaitonSharpRight: "\uE92C",
  notificationLeft: "\uE92D",
  notificationRight: "\uE92E",
  notificationSharpLeft: "\uE92F",
  notificationSlightLeft: "\uE930",
  notificationSlightRight: "\uE931",
  notificationStraight: "\uE932",
  offRampLeft: "\uE933",
  offRampRight: "\uE934",
  offRampSlightLeft: "\uE935",
  offRampSlightRight: "\uE936",
  rampLeft: "\uE933",
  rampRight: "\uE934",
  rampSlightLeft: "\uE935",
  rampSlightRight: "\uE936",
  onRampLeft: "\uE937",
  onRampRight: "\uE938",
  onRampSharpLeft: "\uE939",
  onRampSharpRight: "\uE93A",
  onRampSlightLeft: "\uE93B",
  onRampSlightRight: "\uE93C",
  onRampStraight: "\uE93D",
  rotaryLeft: "\uE93E",
  rotaryRight: "\uE93F",
  rotarySharpLeft: "\uE940",
  rotarySharpRight: "\uE941",
  rotarySlightLeft: "\uE942",
  rotarySlightRight: "\uE943",
  rotaryStraight: "\uE944",
  rotary: "\uE945",
  roundaboutLeft: "\uE946",
  roundaboutRight: "\uE947",
  roundaboutSharpLeft: "\uE948",
  roundaboutSharpRight: "\uE949",
  roundaboutSlightLeft: "\uE94A",
  roundaboutSlightRight: "\uE94B",
  roundaboutStraight: "\uE94C",
  roundabout: "\uE94D",
  turnLeft: "\uE94E",
  turnRight: "\uE94F",
  turnSharpLeft: "\uE950",
  turnSharpRight: "\uE951",
  turnSlightLeft: "\uE952",
  turnSlightRight: "\uE953",
  turnStraight: "\uE954",
  updown: "\uE955"
};

// src/constants/TravelModes.ts
var DRIVING = "DRIVING";
var WALKING = "WALKING";
var TRANSIT = "TRANSIT";
var BICYCLING = "BICYCLING";
var MODE_MAPPING = {
  [DRIVING]: {
    mode: DRIVING,
    name: "Driving",
    icon: NavigationIcons_default.directionsDriving
  },
  [WALKING]: {
    mode: WALKING,
    name: "Walking",
    icon: NavigationIcons_default.directionsWalk
  },
  [TRANSIT]: {
    mode: TRANSIT,
    name: "Transit",
    icon: NavigationIcons_default.directionsTransit
  },
  [BICYCLING]: {
    mode: BICYCLING,
    name: "Bicycling",
    icon: NavigationIcons_default.directionsBike
  }
};
var DEFAULT_MODES = [DRIVING, WALKING, TRANSIT, BICYCLING];
var TravelModes_default = { DRIVING, WALKING, TRANSIT, BICYCLING };
var TravelModeBox = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  render() {
    const options = [];
    this.props.modes?.map((mode) => {
      if (MODE_MAPPING[mode]) {
        options.push(MODE_MAPPING[mode]);
      }
    });
    return /* @__PURE__ */ jsx(
      OptionGroupBox,
      {
        ...this.props,
        fontFamily: this.props.useIcons ? NavigationIconsFont.fontFamily : this.props.fontFamily,
        options,
        useKeyValue: "mode",
        useLabelValue: "icon"
      }
    );
  }
};
/**
 * defaultProps
 * @type {}
 */
TravelModeBox.defaultProps = {
  backgroundColor: "transparent",
  borderColor: "#FFFFFF",
  borderWidth: 1,
  borderRadius: 3,
  contentPadding: 10,
  inverseTextColor: "#FFFFFF",
  defaultValue: DRIVING,
  selected: void 0,
  style: {},
  onChange: void 0,
  theme: void 0,
  invertKeyLabel: false,
  fontSize: 25,
  fontFamily: void 0,
  useIcons: true,
  modes: DEFAULT_MODES
};
var styles_default = (props) => StyleSheet.create({
  /**
   * @travelModeLabelContainer
   */
  travelModeLabelContainer: {
    flexDirection: "row"
  },
  travelModeLabelIcon: {
    ...NavigationIconsFont,
    fontSize: props.size,
    opacity: props.opacity
  },
  travelModeLabelText: {
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: props.fontSize,
    opacity: props.opacity
  }
});
var TravelModeLabel = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {JSX.Element | null}
   */
  render() {
    const styles6 = styles_default(this.props);
    const travelMode = this.props.mode ? MODE_MAPPING[this.props.mode] : void 0;
    if (!travelMode) return null;
    return /* @__PURE__ */ jsxs(View, { style: styles6.travelModeLabelContainer, children: [
      !this.props.useIcon ? null : /* @__PURE__ */ jsx(Text, { style: styles6.travelModeLabelIcon, children: travelMode.icon }),
      !this.props.useLabel ? null : /* @__PURE__ */ jsx(Text, { style: styles6.travelModeLabelText, children: travelMode.name })
    ] });
  }
};
/**
 * defaultProps
 * @type {}
 */
TravelModeLabel.defaultProps = {
  color: void 0,
  opacity: 0.8,
  fontSize: 25,
  fontFamily: void 0,
  useIcon: true,
  useLabel: true,
  mode: void 0
};

// src/themes/Colors.ts
var Colors = {
  // Main brand colors
  primary: "#3273DC",
  // White - app background
  surface: "#F8F9FA",
  // Light gray - cards, surfaces
  text: "#333333",
  // Dark gray - primary text
  textSecondary: "#718096",
  // Medium gray - secondary text
  border: "#E2E8F0"};

// src/components/DirectionsListView/styles.ts
var styles_default2 = (props) => StyleSheet.create({
  /**
   * @directionDetailHeader
   */
  bold: {
    flexWrap: "wrap",
    fontFamily: props.fontFamilyBold || props.fontFamily,
    fontSize: 16,
    fontWeight: "bold"
  },
  directionDetailContent: {
    flexDirection: "column",
    flex: 1
  },
  directionDetailHeader: {
    backgroundColor: Colors.surface,
    flexDirection: "column",
    padding: 25,
    paddingBottom: 10
  },
  directionDetailHeaderAddressLabel: {
    fontFamily: props.fontFamily,
    fontSize: 13,
    fontWeight: "bold",
    opacity: 0.7
  },
  directionDetailHeaderAddressText: {
    fontFamily: props.fontFamily,
    fontSize: 16
  },
  directionDetailHeaderSection: {
    flexDirection: "column",
    marginBottom: 15
  },
  directionDetailIconContainer: {
    alignItems: "center",
    flex: 0,
    justifyContent: "flex-start",
    width: 50
  },
  directionDetailSection: {
    borderColor: Colors.border,
    borderTopWidth: 1,
    flexDirection: "row",
    flex: 1,
    marginBottom: 20,
    paddingTop: 20
  },
  directionDetailSectionContainer: {
    flex: 1,
    margin: 25,
    marginTop: 0
  },
  /**
   * @directionDetailTravel
   */
  directionDetailTravel: {
    margin: 25
  },
  directionDetailTravelDistance: {
    fontFamily: props.fontFamily,
    fontSize: 22,
    opacity: 0.8
  },
  directionDetailTravelDuration: {
    color: Colors.primary,
    fontFamily: props.fontFamily,
    fontSize: 32
  },
  durationDistance: {
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: 14,
    opacity: 0.8
  },
  extra: {
    color: Colors.primary,
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: 13
  },
  regular: {
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: 16
  }
});
var styles = (props) => StyleSheet.create({
  /**
   * @maneuverView
   */
  maneuverArrow: {
    color: props.color,
    fontFamily: "Navigation",
    fontSize: props.size,
    opacity: props.opacity,
    textAlign: "center"
  }
});
var styles_default3 = styles;

// src/constants/DirectionTypes.ts
var DEFAULT_DIRECTION_TYPE = "invalid";
var DirectionTypes_default = {
  NORTH: "NORTH",
  NORTHEAST: "NORTHEAST",
  EAST: "EAST",
  SOUTHEAST: "SOUTHEAST",
  SOUTH: "SOUTH",
  SOUTHWEST: "SOUTHWEST",
  WEST: "WEST",
  NORTHWEST: "NORTHWEST"
};
var ManeuverArrow = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {XML}
   */
  render() {
    const styles6 = styles_default3(this.props);
    const icon = this.props.maneuver && (this.props.maneuver.name || DEFAULT_DIRECTION_TYPE);
    const iconToUse = icon || DEFAULT_DIRECTION_TYPE;
    return /* @__PURE__ */ jsx(Text, { style: styles6.maneuverArrow, children: NavigationIcons_default[iconToUse] });
  }
};
/**
 * defaultProps
 */
ManeuverArrow.defaultProps = {
  maneuver: void 0,
  size: 25,
  opacity: 1,
  color: "#000000"
};
var styles2 = (props) => StyleSheet.create({
  bold: {
    color: props.fontColor || props.color || Colors.text,
    flexWrap: "wrap",
    fontFamily: props.fontFamilyBold || props.fontFamily,
    fontSize: props.fontSize,
    fontWeight: "bold"
  },
  durationDistance: {
    color: Colors.textSecondary,
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: props.fontSize ? props.fontSize * 0.8 : void 0,
    opacity: 0.8
  },
  extra: {
    color: Colors.primary,
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: props.fontSize ? props.fontSize * 0.8 : void 0,
    marginTop: 4
  },
  /**
   * @maneuverLabel
   */
  maneuverLabel: {
    flexDirection: "row"
  },
  /**
   * @fonts
   */
  regular: {
    color: props.fontColor || props.color || Colors.text,
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: props.fontSize
  }
});
var styles_default4 = styles2;
var ManeuverLabel = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * getParsedInstructions
   * @param styles
   * @returns {*}
   */
  getParsedInstructions(styles6) {
    const parts = [];
    const regex = /(\w+)|<(.*?)>(.*?)<\/.*?>/g;
    const mapping = {
      r: styles6.regular,
      b: styles6.bold,
      d: styles6.durationDistance,
      div: styles6.extra
    };
    let m;
    let last = false;
    while (m = regex.exec(this.props.instructions)) {
      if (m.index === regex.lastIndex) {
        regex.lastIndex++;
        last = true;
      }
      if (m[2]) {
        const tag = m[2].split(" ")[0];
        if (tag === "div") m[3] = "\n" + m[3];
        parts.push(
          /* @__PURE__ */ jsxs(Text, { style: mapping[tag], children: [
            m[3],
            last ? "." : " "
          ] }, m.index)
        );
      } else {
        parts.push(
          /* @__PURE__ */ jsxs(Text, { style: mapping.r, children: [
            m[0],
            last ? "." : " "
          ] }, m.index)
        );
      }
    }
    return /* @__PURE__ */ jsx(Text, { style: styles6.regular, children: parts });
  }
  /**
   * render
   * @returns {XML}
   */
  render() {
    const styles6 = styles_default4(this.props);
    return /* @__PURE__ */ jsx(View, { style: styles6.maneuverLabel, children: this.getParsedInstructions(styles6) });
  }
};
/**
 * defaultProps
 */
ManeuverLabel.defaultProps = {
  instructions: "",
  fontFamily: void 0,
  fontFamilyBold: void 0,
  fontSize: 15,
  fontColor: void 0
};
var styles3 = (props) => StyleSheet.create({
  /**
   * @durationDistanceText
   */
  durationDistanceText: {
    flexWrap: "wrap",
    fontFamily: props.fontFamily,
    fontSize: props.fontSize * 0.8,
    opacity: props.opacity
  },
  durationDistanceTravelModeIcon: {
    ...NavigationIconsFont,
    fontSize: props.fontSize * 0.8,
    marginRight: 8,
    opacity: props.opacity
  }
});
var styles_default5 = styles3;
var DurationDistanceLabel = class extends Component {
  /**
   * @constructor
   * @param props Component props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns React element
   */
  render() {
    const styles6 = styles_default5(this.props);
    const travelMode = this.props.mode ? MODE_MAPPING[this.props.mode] : void 0;
    return /* @__PURE__ */ jsxs(Text, { style: [styles6.durationDistanceText, this.props.style], children: [
      !this.props.withTravelModeIcon || !travelMode ? null : /* @__PURE__ */ jsxs(Text, { style: styles6.durationDistanceTravelModeIcon, children: [
        travelMode.icon,
        " "
      ] }),
      this.props.distance ? this.props.distance.text : "",
      this.props.duration ? ["  (", this.props.duration.text, ")"].join("") : ""
    ] });
  }
};
/**
 * Default props for the component
 */
DurationDistanceLabel.defaultProps = {
  style: {},
  fontFamily: "",
  fontSize: 16,
  distance: void 0,
  duration: void 0,
  opacity: 0.8,
  withTravelModeIcon: false
};
var DirectionListViewItem = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {JSX.Element}
   */
  render() {
    const styles6 = styles_default2({
      fontFamily: this.props.fontFamily || "System",
      fontFamilyBold: this.props.fontFamilyBold || this.props.fontFamily || "System"
    });
    return /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailSection, children: [
      /* @__PURE__ */ jsx(View, { style: styles6.directionDetailIconContainer, children: /* @__PURE__ */ jsx(ManeuverArrow, { ...this.props, size: 24 }) }),
      /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailContent, children: [
        /* @__PURE__ */ jsx(ManeuverLabel, { ...this.props }),
        /* @__PURE__ */ jsx(
          DurationDistanceLabel,
          {
            ...this.props,
            withTravelModeIcon: this.props.displayTravelMode,
            distance: this.props.distance ? {
              text: this.props.distance.text,
              value: this.props.distance.value || 0
            } : void 0,
            duration: this.props.duration ? {
              text: this.props.duration.text,
              value: this.props.duration.value || 0
            } : void 0
          }
        )
      ] })
    ] });
  }
};
/**
 * defaultProps
 */
DirectionListViewItem.defaultProps = {
  instructions: "",
  fontFamily: void 0,
  fontFamilyBold: void 0,
  distance: void 0,
  duration: void 0,
  maneuver: void 0,
  fontSize: void 0,
  displayTravelMode: false
};
var DirectionsListView = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {JSX.Element | null}
   */
  render() {
    const steps = this.props.route ? this.props.route.steps : false;
    if (!steps || steps.constructor !== Array) return null;
    const styles6 = styles_default2({
      fontFamily: this.props.fontFamily || "System",
      fontFamilyBold: this.props.fontFamilyBold || this.props.fontFamily || "System"
    });
    return /* @__PURE__ */ jsxs(ScrollView, { children: [
      !this.props.showOriginDestinationHeader ? null : /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailHeader, children: [
        /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailHeaderSection, children: [
          /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailHeaderAddressLabel, children: "FROM" }),
          /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailHeaderAddressText, children: this.props.route.origin.address })
        ] }),
        /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailHeaderSection, children: [
          /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailHeaderAddressLabel, children: "TO" }),
          /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailHeaderAddressText, children: this.props.route.destination.address })
        ] })
      ] }),
      /* @__PURE__ */ jsxs(View, { style: styles6.directionDetailTravel, children: [
        /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailTravelDuration, children: this.props.route.duration.text }),
        /* @__PURE__ */ jsx(Text, { style: styles6.directionDetailTravelDistance, children: this.props.route.distance.text })
      ] }),
      /* @__PURE__ */ jsx(View, { style: styles6.directionDetailSectionContainer, children: steps.map((step, index) => /* @__PURE__ */ createElement(DirectionListViewItem, { ...this.props, key: index, ...step })) })
    ] });
  }
};
/**
 * defaultProps
 */
DirectionsListView.defaultProps = {
  fontFamily: void 0,
  fontFamilyBold: void 0,
  showOriginDestinationHeader: true,
  displayTravelMode: false
};

// src/constants/NavigationModes.ts
var NavigationModes_default = {
  IDLE: "IDLE",
  ROUTE: "ROUTE",
  NAVIGATION: "NAVIGATION"
};

// src/themes/index.ts
var defaultThemeSettings = {
  /**
   * @markers
   */
  Markers: {
    ["ORIGIN" /* ORIGIN */]: {
      icon: NavigationIcons_default.place,
      color: "#77dd77",
      fontSize: 40
    },
    ["DESTINATION" /* DESTINATION */]: {
      icon: NavigationIcons_default.place,
      color: "#ff4500",
      fontSize: 40
    },
    ["POSITION_DOT" /* POSITION_DOT */]: {
      icon: NavigationIcons_default.compassDot,
      color: "#387bc1",
      fontSize: 30
    },
    ["POSITION_ARROW" /* POSITION_ARROW */]: {
      icon: NavigationIcons_default.navigate,
      size: 100,
      fontSize: 80,
      color: "#ffffff",
      backgroundColor: "#387bc1"
    }
  },
  Polylines: {
    ["ROUTE" /* ROUTE */]: {
      fillColor: "#00b3fd",
      strokeColor: "#387bc1",
      strokeWidth: 18,
      borderWidth: 4
    },
    ["ROUTE_ALTERNATIVE" /* ROUTE_ALTERNATIVE */]: {
      fillColor: "#cccccc",
      strokeColor: "#a0a0a0",
      strokeWidth: 18,
      borderWidth: 4
    },
    ["PENDING" /* PENDING */]: {
      fillColor: "#00b3fd",
      strokeColor: "#387bc1",
      strokeWidth: 18,
      borderWidth: 4
    },
    ["NEXT" /* NEXT */]: {
      fillColor: "#00b3fd",
      strokeColor: "#387bc1",
      strokeWidth: 18,
      borderWidth: 4
    }
  }
};
var connectTheme = (theme) => {
  return Object.assign({}, defaultThemeSettings, theme || {});
};
var themes_default = connectTheme;
var RoutePolyline = class extends Component {
  /**
   * constructor
   * @param props
   */
  constructor(props) {
    super(props);
    this.theme = themes_default(props.theme).Polylines[this.props.type];
  }
  /**
   * @render
   * @returns {ReactElement[] | null}
   */
  render() {
    if (!this.props.coordinates) return null;
    if (!this.theme) {
      throw new Error(`RoutePolyline does not support type ${this.props.type}.`);
    }
    const components = [
      /* @__PURE__ */ jsx(
        Polyline,
        {
          strokeWidth: this.theme.strokeWidth,
          strokeColor: this.theme.strokeColor,
          coordinates: this.props.coordinates,
          lineCap: "round"
        },
        0
      )
    ];
    if (this.theme.fillColor) {
      const borderWidth = this.theme.strokeWidth - (this.theme.borderWidth || 3);
      components.push(
        /* @__PURE__ */ jsx(
          Polyline,
          {
            strokeWidth: borderWidth,
            strokeColor: this.theme.fillColor,
            coordinates: this.props.coordinates,
            lineCap: "round"
          },
          1
        )
      );
    }
    return components;
  }
};
var styles_default6 = (props) => StyleSheet.create({
  markerText: {
    ...NavigationIconsFont,
    color: props.color || "#000000",
    fontSize: props.fontSize || 30
  }
});
var RouterMarker = class extends Component {
  /**
   * constructor
   * @param props
   */
  constructor(props) {
    super(props);
    this.theme = themes_default(props.theme).Markers[this.props.type];
  }
  /**
   * @render
   * @returns {*}
   */
  render() {
    const styles6 = styles_default6(this.theme);
    return /* @__PURE__ */ jsx(Marker, { coordinate: this.props.coordinate, children: /* @__PURE__ */ jsx(Text, { style: styles6.markerText, children: this.theme.icon }) });
  }
};
var styles4 = (props) => StyleSheet.create({
  positionMarkerArrow: {
    alignItems: "center",
    backgroundColor: props.backgroundColor,
    borderRadius: props.size,
    height: props.size,
    justifyContent: "center",
    transform: [{ rotateX: props.angle + "deg" }],
    width: props.size
  },
  positionMarkerText: {
    ...NavigationIconsFont,
    color: props.color,
    fontSize: props.fontSize
  }
});
var styles_default7 = styles4;
var PositionMarker = class extends Component {
  /**
   * constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @render
   * @returns {*}
   */
  render() {
    if (!this.props.coordinate) return null;
    const type = this.props.type || "POSITION_DOT" /* POSITION_DOT */;
    this.theme = themes_default(this.props.theme).Markers[type];
    const styles6 = styles_default7(Object.assign({}, this.props, this.theme));
    return type === "POSITION_ARROW" /* POSITION_ARROW */ ? this.renderArrow(styles6) : this.renderDot(styles6);
  }
  /**
   * renderArrow
   * @param styles
   * @returns {*}
   */
  renderArrow(styles6) {
    return /* @__PURE__ */ jsx(Marker, { coordinate: this.props.coordinate, flat: false, children: /* @__PURE__ */ jsx(View, { style: styles6.positionMarkerArrow, children: /* @__PURE__ */ jsx(Text, { style: styles6.positionMarkerText, children: this.theme.icon }) }) });
  }
  /**
   * renderDot
   * @param styles
   * @returns {*}
   */
  renderDot(styles6) {
    return /* @__PURE__ */ jsx(Marker, { coordinate: this.props.coordinate, flat: false, children: /* @__PURE__ */ jsx(Text, { style: styles6.positionMarkerText, children: this.theme.icon }) });
  }
};
/**
 * defaultProps
 */
PositionMarker.defaultProps = {
  coordinate: void 0,
  size: 40,
  fontSize: 30,
  type: "POSITION_DOT" /* POSITION_DOT */,
  color: "#252525",
  angle: 60,
  borderWidth: 0,
  borderColor: void 0,
  backgroundColor: "#252525",
  positionMarkerText: "A"
};
var toQueryParams = (object) => {
  return Object.keys(object).filter((key) => !!object[key]).map((key) => key + "=" + encodeURIComponent(object[key])).join("&");
};
var toLatLng = (value) => {
  if (typeof value === "string") return value;
  return value && value.latitude && value.longitude ? `${value.latitude},${value.longitude}` : value;
};
var toCoordinate = (latlng) => {
  const { lat, lng } = latlng;
  return { latitude: lat, longitude: lng };
};
var toArcPolygon = (coordinate, initialBearing, finalBearing, radius) => {
  const points = 32;
  const result = [];
  if (initialBearing > finalBearing) finalBearing += 360;
  let deltaBearing = finalBearing - initialBearing;
  deltaBearing = deltaBearing / points;
  for (let i = 0; i < points + 1; i++) {
    result.push(
      GeoLib2.computeDestinationPoint(coordinate, radius, initialBearing + i * deltaBearing)
    );
  }
  return result;
};
var toNameId = (str, prepend = false, append = false) => {
  str = str.toLowerCase().replace(/_/g, " ").replace(/\b[a-z]/g, (letter) => letter.toUpperCase()).replace(/\s/g, "");
  return (prepend ? prepend : "") + str + (append ? append : "");
};

// src/modules/Geocoder/index.ts
var _Geocoder = class _Geocoder {
  /**
   * Creates a new Geocoder instance
   *
   * @constructor
   * @param {string} apiKey - Google Maps API key
   * @param {GeocoderOptions|false} [options=false] - Additional options for the geocoder
   * @throws {Error} Will throw an error if apiKey is not provided
   */
  constructor(apiKey, options = false) {
    this.apiKey = apiKey;
    this.options = options || {};
  }
  /**
   * Geocodes an address or reverse geocodes coordinates
   *
   * @async
   * @param {...any[]} params - Either a string address or an object with query parameters
   * @returns {Promise<LocationResult[]>} A promise that resolves to an array of location results
   * @throws {Error} Will reject if the API returns a non-OK status or if there's a network error
   *
   * @example
   * // With a string address
   * geocoder.getFromLocation('1600 Amphitheatre Parkway, Mountain View, CA')
   *   .then(results => console.log(results));
   *
   * // With query parameters
   * geocoder.getFromLocation({
   *   address: '1600 Amphitheatre Parkway',
   *   components: 'country:US'
   * })
   *   .then(results => console.log(results));
   */
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  async getFromLocation(...params) {
    const queryParams = params.length === 1 && typeof params[0] === "string" ? { address: params[0] } : params[0];
    if (!params) return Promise.reject("Not enough parameters");
    queryParams.key = this.apiKey;
    if (this.options.language) queryParams.language = this.options.language;
    const url = `https://maps.google.com/maps/api/geocode/json?${toQueryParams(queryParams)}`;
    let response, data;
    try {
      response = await fetch(url);
    } catch (error) {
      return Promise.reject(error);
    }
    try {
      data = await response.json();
    } catch (error) {
      return Promise.reject(error);
    }
    if (data.status !== "OK") {
      return Promise.reject(data);
    }
    return data.results;
  }
  /**
   * Reverse geocodes coordinates to get address information
   *
   * @param {number} lat - Latitude in decimal degrees
   * @param {number} lng - Longitude in decimal degrees
   * @returns {Promise<LocationResult[]>} A promise that resolves to an array of location results
   * @throws {Error} Will reject if the API returns a non-OK status or if there's a network error
   *
   * @example
   * geocoder.getFromLatLng(37.4224764, -122.0842499)
   *   .then(results => console.log(results));
   */
  getFromLatLng(lat, lng) {
    return this.getFromLocation({ latlng: `${lat},${lng}` });
  }
  /**
   * Converts Google's geocoding results into a simplified format
   *
   * @param {LocationResult[]} results - The results from getFromLocation or getFromLatLng
   * @returns {MinimizedResult[]} An array of simplified location results
   *
   * @example
   * geocoder.getFromLocation('1600 Amphitheatre Parkway, Mountain View, CA')
   *   .then(results => {
   *     const minimized = geocoder.minimizeResults(results);
   *     console.log(minimized);
   *   });
   */
  minimizeResults(results) {
    if (results.constructor != Array) return [];
    return results.map((result) => {
      const { lat, lng } = result.geometry.location;
      return {
        components: this.minimizeAddressComponents(result.address_components),
        address: result.formatted_address,
        coordinate: {
          latitude: lat,
          longitude: lng
        }
      };
    });
  }
  /**
   * Simplifies Google's address components into a more manageable structure
   *
   * @param {AddressComponent[]} components - The address components from a geocoding result
   * @returns {MinimizedAddressComponents} A simplified representation of the address components
   *
   * @example
   * // Result example:
   * {
   *   street: { short: "5th Ave", long: "Fifth Avenue" },
   *   city: { short: "NY", long: "New York" },
   *   state: { short: "NY", long: "New York" },
   *   country: { short: "US", long: "United States" }
   * }
   */
  minimizeAddressComponents(components) {
    const results = {};
    const ids = Object.keys(_Geocoder.AddressComponentMapping);
    components.forEach((component) => {
      const index = ids.indexOf(component.types[0]);
      if (index != -1) {
        results[_Geocoder.AddressComponentMapping[ids[index]]] = {
          short: component.short_name,
          long: component.long_name
        };
      }
    });
    return results;
  }
};
/**
 * Mapping between Google's address component types and our simplified naming schema
 *
 * @static
 * @type {AddressComponentMappingType}
 * @example
 * {
 *   street_number: 'number',  // Maps Google's 'street_number' to our 'number'
 *   route: 'street',          // Maps Google's 'route' to our 'street'
 *   // ...and so on
 * }
 */
_Geocoder.AddressComponentMapping = {
  street_number: "number",
  route: "street",
  postal_code: "zip",
  country: "country",
  locality: "city",
  administrative_area_level_1: "state",
  administrative_area_level_2: "county"
};
var Geocoder = _Geocoder;
var Directions = class {
  /**
   * Creates a new Directions instance to fetch and parse routing information
   *
   * @constructor
   * @param {string} apiKey - Google Maps API key
   * @param {DirectionsOptions|false} [options=false] - Additional options for the directions service
   */
  constructor(apiKey, options = false) {
    this.apiKey = apiKey;
    this.options = options || {};
  }
  /**
   * Fetches routing information between an origin and destination
   *
   * @async
   * @param {string|Coordinate} origin - Starting point as address string or coordinates
   * @param {string|Coordinate} destination - Ending point as address string or coordinates
   * @param {DirectionsOptions|false} [options=false] - Additional options for this specific request
   * @returns {Promise<Route[]>} Promise resolving to an array of routes
   * @throws {Error} Will reject if the API returns a non-OK status or if there's a network error
   *
   * @example
   * // Fetch directions with coordinates
   * directions.fetch(
   *   { latitude: 40.7128, longitude: -74.0060 }, // New York
   *   { latitude: 42.3601, longitude: -71.0589 }  // Boston
   * ).then(routes => console.log(routes));
   *
   * @example
   * // Fetch directions with addresses and options
   * directions.fetch(
   *   'Chicago, IL',
   *   'St. Louis, MO',
   *   {
   *     mode: TravelModes.DRIVING,
   *     alternatives: true,
   *     avoid: 'tolls'
   *   }
   * ).then(routes => console.log(routes));
   */
  fetch(origin, destination, options = false) {
    const routeOptions = Object.assign(
      {
        key: this.apiKey,
        mode: TravelModes_default.DRIVING
      },
      this.options,
      options || {}
    );
    const queryParams = {
      origin: toLatLng(origin),
      destination: toLatLng(destination),
      ...routeOptions
    };
    if (queryParams.mode) queryParams.mode = queryParams.mode.toLowerCase();
    const url = `https://maps.google.com/maps/api/directions/json?${toQueryParams(queryParams)}`;
    return fetch(url).then((response) => response.json()).then((json) => {
      if (json.status !== "OK") {
        const errorMessage = json.error_message || "Unknown error";
        return Promise.reject(errorMessage);
      }
      return this.parse(json);
    });
  }
  /**
   * Parses the raw Google Directions API response into a more usable format
   *
   * @param {DirectionsResponse} json - The raw response from the Directions API
   * @returns {Route[]} An array of parsed routes
   *
   * @example
   * fetch('https://maps.google.com/maps/api/directions/json?...')
   *   .then(response => response.json())
   *   .then(json => {
   *     if (json.status === 'OK') {
   *       const routes = directions.parse(json);
   *       console.log(routes);
   *     }
   *   });
   */
  parse(json) {
    if (!json.routes.length) return [];
    return json.routes.map((route) => {
      if (!route.legs.length) return null;
      const leg = route.legs[0];
      const markers = [
        // origin
        {
          coordinate: toCoordinate(leg.start_location),
          type: "ORIGIN" /* ORIGIN */
        },
        // destination
        {
          coordinate: toCoordinate(leg.end_location),
          type: "DESTINATION" /* DESTINATION */
        }
      ];
      const steps = leg.steps.map(
        (step, index) => this.parseStep(step, leg.steps[index + 1] ? leg.steps[index + 1] : false)
      );
      steps.push({
        final: true,
        bearing: steps[steps.length - 1].bearing,
        compass: steps[steps.length - 1].compass,
        start: steps[steps.length - 1].start,
        end: false,
        maneuver: {
          name: "flag",
          type: "flag"
        },
        polyline: {
          coordinates: [],
          type: "ROUTE" /* ROUTE */
        },
        instructions: leg.end_address,
        duration: steps[steps.length - 1].duration,
        distance: steps[steps.length - 1].distance
      });
      const polylines = steps.map((step) => step.polyline);
      const boundingBox = [
        toCoordinate(route.bounds.northeast),
        toCoordinate(route.bounds.southwest)
      ];
      return {
        title: route.summary,
        markers,
        steps,
        polylines,
        bounds: {
          boundingBox,
          center: GeoLib2.getCenter(boundingBox),
          northEast: toCoordinate(route.bounds.northeast),
          southWest: toCoordinate(route.bounds.southwest)
        },
        initialBearing: steps.length ? steps[0].bearing : 0,
        duration: leg.duration,
        distance: leg.distance,
        origin: {
          address: leg.start_address,
          latlng: leg.start_location,
          coordinate: toCoordinate(leg.start_location)
        },
        destination: {
          address: leg.end_address,
          latlng: leg.end_location,
          coordinate: toCoordinate(leg.end_location)
        }
      };
    }).filter((route) => route !== null);
  }
  /**
   * Parses a single step from the Directions API response
   *
   * @param {Step} step - The step to parse
   * @param {Step|false} nextStep - The next step in the route, or false if this is the last step
   * @returns {Step} The parsed step with additional navigation information
   */
  parseStep(step, nextStep) {
    const bearing = GeoLib2.getGreatCircleBearing(
      toCoordinate(step.start_location),
      toCoordinate(nextStep ? nextStep.start_location : step.end_location)
    );
    const compass = this.decodeCompass(bearing);
    return {
      compass: compass === false ? void 0 : compass,
      maneuver: this.decodeManeuver(step),
      bearing,
      mode: step.travel_mode,
      start: toCoordinate(step.start_location),
      end: toCoordinate(step.end_location),
      polyline: {
        coordinates: this.decodePolylineToCoordinates(step.polyline.points || "", 5),
        type: "ROUTE" /* ROUTE */
      },
      duration: step.duration,
      distance: step.distance,
      instructions: step.html_instructions || ""
    };
  }
  /**
   * Decodes the maneuver type and name from a step
   *
   * @param {Step} step - The step containing maneuver information
   * @returns {Maneuver} The decoded maneuver with name and type
   *
   * @example
   * // For a step with maneuver.type = "turn-right"
   * const maneuver = directions.decodeManeuver(step);
   * console.log(maneuver.name); // "turnRight"
   * console.log(maneuver.type); // "turn-right"
   */
  decodeManeuver(step) {
    const maneuver = step.maneuver?.type || DEFAULT_DIRECTION_TYPE;
    const name = maneuver.split("-").map((d, i) => i == 0 ? d : d[0].toUpperCase() + d.slice(1)).join("");
    return {
      name,
      type: maneuver
    };
  }
  /**
   * Decodes a bearing into compass directions
   *
   * @param {number} bearing - The bearing in degrees from north (0-360)
   * @returns {Compass|false} The compass directions or false if bearing is negative
   *
   * @example
   * const compass = directions.decodeCompass(45);
   * console.log(compass.detail);  // "northeast"
   * console.log(compass.simple);  // "north"
   */
  decodeCompass(bearing) {
    if (bearing < 0) return false;
    const compass = [
      DirectionTypes_default.NORTH,
      DirectionTypes_default.NORTHEAST,
      DirectionTypes_default.EAST,
      DirectionTypes_default.SOUTHEAST,
      DirectionTypes_default.SOUTH,
      DirectionTypes_default.SOUTHWEST,
      DirectionTypes_default.WEST,
      DirectionTypes_default.NORTHWEST
    ];
    const compassSimple = [
      DirectionTypes_default.NORTH,
      DirectionTypes_default.EAST,
      DirectionTypes_default.SOUTH,
      DirectionTypes_default.WEST
    ];
    const calc = (a) => {
      const c = Math.ceil(bearing / (360 / a.length)) - 1;
      return a[c < 0 ? 0 : c > a.length - 1 ? a.length - 1 : c];
    };
    return {
      detail: calc(compass),
      simple: calc(compassSimple)
    };
  }
  /**
   * Decodes an encoded polyline string into an array of coordinates
   *
   * @param {string} t - The encoded polyline string
   * @param {number} [e=5] - The precision of the encoding
   * @returns {Coordinate[]} Array of decoded coordinates
   *
   * @example
   * // Decode a polyline string
   * const coordinates = directions.decodePolylineToCoordinates("_p~iF~ps|U_ulLnnqC_mqNvxq`@");
   * console.log(coordinates[0]); // { latitude: 38.5, longitude: -120.2 }
   */
  decodePolylineToCoordinates(t, e = 5) {
    let n, o, u = 0, l = 0, r = 0;
    const d = [];
    let h = 0, i = 0, a = null;
    const c = Math.pow(10, e || 5);
    while (u < t.length) {
      a = null;
      h = 0;
      i = 0;
      do {
        a = t.charCodeAt(u++) - 63;
        i |= (31 & a) << h;
        h += 5;
      } while (a >= 32);
      n = 1 & i ? ~(i >> 1) : i >> 1;
      h = 0;
      i = 0;
      do {
        a = t.charCodeAt(u++) - 63;
        i |= (31 & a) << h;
        h += 5;
      } while (a >= 32);
      o = 1 & i ? ~(i >> 1) : i >> 1;
      l += n;
      r += o;
      d.push([l / c, r / c]);
    }
    return d.map(function(t2) {
      return {
        latitude: t2[0],
        longitude: t2[1]
      };
    });
  }
};
var EVENTS = {
  ENTERING_ON_COURSE: "ENTERING_ON_COURSE",
  ENTERING_OFF_COURSE: "ENTERING_OFF_COURSE",
  INSIDE: "INSIDE",
  INSIDE_CENTER: "INSIDE_CENTER",
  LEAVING: "LEAVING",
  LEAVING_ON_COURSE: "LEAVING_ON_COURSE",
  LEAVING_OFF_COURSE: "LEAVING_OFF_COURSE"
};
var STATES = {
  OUTSIDE: "OUTSIDE",
  ENTERED: "ENTERED",
  INSIDE: "INSIDE",
  CENTER: "CENTER",
  LEAVING: "LEAVING",
  LEFT: "LEFT",
  EXPIRED: "EXPIRED"
};
var TYPES = {
  CIRCLE: "CIRCLE",
  STEP: "STEP"
};
var TrapTypes_default = {
  Events: EVENTS,
  States: STATES,
  Types: TYPES};

// src/modules/Traps/index.ts
var Traps = class {
  /**
   * Creates a new Traps instance
   * @param instance - Parent instance reference
   */
  constructor() {
    this.traps = {};
    this.counter = 0;
  }
  /**
   * Processes a new position update, checking all traps against it
   * @param position - Current position with coordinates and heading
   */
  execute(position) {
    const { coordinate, heading } = position;
    this.__matches(coordinate, heading);
  }
  /**
   * Adds a new trap to the collection
   * @param trap - Trap configuration
   * @param callback - Optional callback function for trap events
   * @returns The created trap
   */
  add(trap, callback) {
    this.counter++;
    const counter = this.counter;
    this.traps[counter] = Object.assign({}, trap, {
      index: counter,
      state: TrapTypes_default.States.OUTSIDE,
      callback
    });
    Object.keys(TrapTypes_default.States).forEach((state) => {
      const propName = toNameId(state, "is");
      this.traps[counter][propName] = function() {
        return this.state === state;
      };
    });
    return this.traps[counter];
  }
  /**
   * Returns an array of all traps
   * @returns Array of all traps
   */
  getArray() {
    return Object.keys(this.traps).map((id) => this.traps[Number(id)]);
  }
  /**
   * Creates a circular trap around a coordinate
   * @param coordinate - Center coordinate of the trap
   * @param radius - Radius in meters
   * @param options - Additional options
   * @param callback - Optional callback function for trap events
   * @returns The created trap
   */
  watchRadius(coordinate, radius, options, callback) {
    return this.add(
      {
        type: TrapTypes_default.Types.CIRCLE,
        coordinate,
        radius,
        options
      },
      callback
    );
  }
  /**
   * Creates a step trap for navigation monitoring
   * @param step - The current navigation step
   * @param nextStep - The next navigation step (optional)
   * @param options - Configuration options for the trap
   * @param callback - Optional callback function for trap events
   * @returns The created trap
   */
  watchStep(step, nextStep, options, callback) {
    options = Object.assign(
      {},
      {
        distance: 15,
        innerRadiusTolerance: 0.75,
        centerRadiusTolerance: 0.5,
        courseTolerance: 30
      },
      options
    );
    const distanceToNextPoint = options.distance || step.distance.value;
    const coordinate = step.start;
    return this.add(
      {
        type: TrapTypes_default.Types.STEP,
        innerRadius: distanceToNextPoint * options.innerRadiusTolerance,
        centerRadius: distanceToNextPoint * options.centerRadiusTolerance,
        outerRadius: distanceToNextPoint,
        courseTolerance: options.courseTolerance,
        coordinate,
        step,
        nextStep
      },
      callback
    );
  }
  /**
   * Updates the state of a trap and triggers its callback
   * @param trap - The trap to update
   * @param event - The event that triggered the state change
   * @param state - The new state to set
   */
  nextState(trap, event, state) {
    this.traps[trap.index].state = state;
    if (event.constructor == String && trap.callback) {
      trap.callback(trap, event, state);
    }
  }
  /**
   * Checks all traps against the current position and processes state changes
   * @param coordinate - Current geographic coordinates
   * @param heading - Current heading in degrees
   * @returns Array of trap match results
   * @private
   */
  __matches(coordinate, heading) {
    const traps = Object.keys(this.traps);
    return traps.map((index) => {
      const trap = this.traps[Number(index)];
      if (trap.state != TrapTypes_default.States.EXPIRED) {
        switch (trap.type) {
          case TrapTypes_default.Types.CIRCLE:
            if (GeoLib2.isPointWithinRadius(coordinate, trap.coordinate, trap.radius)) {
              return true;
            }
            break;
          case TrapTypes_default.Types.STEP: {
            const insideOuter = GeoLib2.isPointWithinRadius(
              coordinate,
              trap.coordinate,
              trap.outerRadius
            );
            const insideInner = GeoLib2.isPointWithinRadius(
              coordinate,
              trap.coordinate,
              trap.innerRadius
            );
            const stateMap = {
              [TrapTypes_default.States.OUTSIDE]: [
                TrapTypes_default.States.ENTERED,
                () => {
                  const isWithinCourse = this.isWithinCourse(
                    trap.step.bearing,
                    heading,
                    trap.courseTolerance
                  );
                  return insideOuter ? isWithinCourse ? TrapTypes_default.Events.ENTERING_ON_COURSE : TrapTypes_default.Events.ENTERING_OFF_COURSE : false;
                }
              ],
              [TrapTypes_default.States.ENTERED]: [
                TrapTypes_default.States.INSIDE,
                () => {
                  return insideOuter ? TrapTypes_default.Events.INSIDE : false;
                }
              ],
              [TrapTypes_default.States.INSIDE]: [
                TrapTypes_default.States.CENTER,
                () => {
                  return insideInner ? TrapTypes_default.Events.INSIDE_CENTER : false;
                }
              ],
              [TrapTypes_default.States.CENTER]: [
                TrapTypes_default.States.LEAVING,
                () => {
                  const isWithinCourse = this.isWithinCourse(
                    trap.nextStep ? trap.nextStep.bearing : trap.step.bearing,
                    heading,
                    trap.courseTolerance
                  );
                  return insideOuter && !insideInner ? isWithinCourse ? TrapTypes_default.Events.LEAVING_ON_COURSE : TrapTypes_default.Events.LEAVING_OFF_COURSE : false;
                }
              ],
              [TrapTypes_default.States.LEAVING]: [
                TrapTypes_default.States.LEFT,
                () => {
                  return !insideOuter && !insideInner ? TrapTypes_default.Events.LEAVING : false;
                }
              ],
              [TrapTypes_default.States.LEFT]: [
                TrapTypes_default.States.EXPIRED,
                () => {
                  return true;
                }
              ]
            };
            if (stateMap[trap.state]) {
              const func = stateMap[trap.state];
              const eventResult = func[1]();
              if (eventResult) {
                const eventString = typeof eventResult === "boolean" ? "true" : eventResult;
                this.nextState(trap, eventString, func[0]);
                return true;
              }
            }
            break;
          }
        }
      }
      return void 0;
    });
  }
  /**
   * Determines if a heading is within tolerance of a given bearing
   * @param bearing - Target bearing in degrees
   * @param heading - Current heading in degrees
   * @param tolerance - Tolerance in degrees
   * @returns True if the heading is within tolerance of the bearing
   */
  isWithinCourse(bearing, heading, tolerance = 0) {
    const low = bearing - tolerance;
    const high = bearing + tolerance;
    return (low < 0 && heading > 360 - -1 * low || heading > low) && (high > 360 && heading < high - 360 || heading < high);
  }
};
var Simulator = class {
  /**
   * constructor
   * @param instance The simulator instance
   */
  constructor(instance) {
    this.pointIndex = 0;
    this.points = [];
    this.lastBearing = false;
    this.instance = instance;
    this.speed = 30;
    this.turnSpeed = 700;
  }
  /**
   * start
   * @param route The route to simulate
   */
  start(route) {
    this.pointIndex = 0;
    const steps = route.steps;
    const points = [];
    const result = [];
    steps.map(
      (step) => step.polyline.coordinates.map((coordinate) => points.push(Object.assign({}, coordinate)))
    );
    points.forEach((point, index) => {
      const nextPoint = points[index + 1];
      if (nextPoint && !nextPoint.final === true) {
        const distance = Math.round(GeoLib2.getDistance(point, nextPoint));
        const bearing = GeoLib2.getGreatCircleBearing(point, nextPoint);
        if (bearing !== 0) {
          if (distance > 1) {
            for (let x = 1; x < distance; x++) {
              result.push(
                Object.assign({}, { bearing }, GeoLib2.computeDestinationPoint(point, x, bearing))
              );
            }
          } else {
            result.push(Object.assign({}, { bearing }, point));
          }
        }
      }
    });
    this.pointIndex = 0;
    this.points = result;
    this.lastBearing = false;
    this.drive();
  }
  drive() {
    const point = this.points[this.pointIndex];
    let speed = this.speed;
    if (point && point.bearing) {
      let allowPositionUpdate = true;
      if (this.lastBearing !== false && point.bearing !== void 0) {
        if (point.bearing > this.lastBearing - 10 && point.bearing < this.lastBearing + 10) {
          this.instance.updateBearing(point.bearing, this.turnSpeed);
        } else {
          allowPositionUpdate = false;
          speed = this.turnSpeed;
          this.instance.updateBearing(point.bearing, this.turnSpeed);
        }
        this.lastBearing = point.bearing;
      }
      if (allowPositionUpdate) {
        this.instance.setPosition({
          ...point,
          heading: point.bearing,
          coordinate: {
            latitude: point.latitude,
            longitude: point.longitude
          }
        });
        this.pointIndex++;
      }
      setTimeout(() => this.drive(), speed);
    }
  }
};
var customLogger = logger.createLogger({
  levels: {
    debug: 0,
    info: 1,
    warn: 2,
    error: 3
  },
  severity: "debug",
  transport: consoleTransport,
  transportOptions: {
    colors: {
      info: "blueBright",
      warn: "yellowBright",
      error: "redBright"
    }
  },
  async: true,
  dateFormat: "time",
  printLevel: true,
  printDate: true,
  fixedExtLvlLength: false,
  enabled: true
});
var Logger_default = customLogger.extend("react-native-maps-navigation-ts");
var MapViewNavigation = class extends Component {
  /**
   * @constructor
   * @param props Component props
   */
  constructor(props) {
    super(props);
    this.stepIndex = 0;
    this.geoCoder = new Geocoder(this.props.apiKey, {
      language: this.props.language
    });
    this.directionsCoder = new Directions(this.props.apiKey, {
      language: this.props.language
    });
    this.traps = new Traps();
    this.state = {
      route: false,
      markers: [],
      position: {
        coordinate: {
          latitude: 0,
          longitude: 0
        },
        heading: 0
      },
      navigationMode: NavigationModes_default.IDLE,
      travelMode: TravelModes_default.DRIVING,
      stepIndex: false
    };
    this.theme = themes_default(this.props.theme);
    const { width, height } = Dimensions.get("window");
    this.aspectRatio = width / height;
  }
  /**
   * @componentDidMount
   */
  componentDidMount() {
    this.watchId = geolocation.watchPosition((position) => {
      this.setPosition({
        coordinate: {
          latitude: position.coords.latitude,
          longitude: position.coords.longitude
        },
        heading: position.coords.heading
      });
    });
  }
  /**
   * @componentWillUnmount
   */
  componentWillUnmount() {
    geolocation.clearWatch(this.watchId);
  }
  /**
   * @componentDidUpdate
   * @param prevProps Previous props
   */
  componentDidUpdate(prevProps) {
    if (this.props.origin && this.props.destination) {
      if (prevProps.navigationMode !== this.props.navigationMode || prevProps.travelMode !== this.props.travelMode || prevProps.origin !== this.props.origin || prevProps.destination !== this.props.destination) {
        this.updateRoute();
      }
    }
  }
  /**
   * getCoordinates
   * @param address Address to geocode
   * @returns {Promise<Array>} Coordinates
   */
  getCoordinates(address) {
    return this.geoCoder.getFromLocation(address).then((results) => {
      const coordinates = this.geoCoder.minimizeResults(results);
      return coordinates.length === 1 ? coordinates[0] : coordinates;
    });
  }
  /**
   * getCoordinatesRaw
   * @param address Address to geocode
   * @returns {Promise<Array>} Raw coordinates
   */
  getCoordinatesRaw(address) {
    return this.geoCoder.getFromLocation(address);
  }
  /**
   * getZoomValue
   * @param level Zoom level
   * @returns Region delta values
   */
  getZoomValue(level) {
    const value = 1e-5 * (this.props.maxZoom - (level < this.props.minZoom ? this.props.minZoom : level));
    return {
      latitudeDelta: value,
      longitudeDelta: value * this.aspectRatio
    };
  }
  /**
   * getBoundingBoxZoomValue
   * @param b Bounding box coordinates
   * @param quantifier Zoom quantifier
   * @returns Region delta values
   */
  getBoundingBoxZoomValue(b, quantifier = 1) {
    if (b.length !== 2) return null;
    const latitudeDelta = (b[0].latitude > b[1].latitude ? b[0].latitude - b[1].latitude : b[1].latitude - b[0].latitude) * quantifier;
    return {
      latitudeDelta,
      longitudeDelta: latitudeDelta * this.aspectRatio
    };
  }
  /**
   * updatePosition
   * @param coordinate Coordinate to update to
   * @param duration Animation duration
   */
  updatePosition(coordinate, duration = 0.5) {
    if (this.props.map) {
      this.props.map().animateCamera(
        {
          center: coordinate,
          zoom: this.props.navigationZoomLevel,
          heading: this.state.position.heading,
          pitch: this.props.navigationViewingAngle
        },
        {
          duration
        }
      );
    }
  }
  /**
   * updateBearing
   * @param bearing New bearing
   * @param duration Animation duration
   */
  updateBearing(bearing, duration = 0.5) {
    if (this.props.map) {
      this.props.map().animateToRegion(
        {
          latitude: this.state.position.coordinate.latitude,
          longitude: this.state.position.coordinate.longitude,
          latitudeDelta: this.props.navigationZoomLevel,
          longitudeDelta: this.props.navigationZoomLevel * this.aspectRatio
        },
        duration
      );
    }
  }
  /**
   * updateStep
   * @param stepIndex Index of the step to update to
   */
  updateStep(stepIndex = 0) {
    if (!this.state.route) return;
    const step = this.state.route.steps[stepIndex < 0 ? 0 : stepIndex];
    const nextStep = this.state.route.steps[stepIndex + 1];
    if (this.props.onStepChange) {
      this.props.onStepChange(step, nextStep);
    }
    this.traps.watchStep(
      step,
      nextStep,
      {
        distance: this.props.routeStepDistance,
        innerRadiusTolerance: this.props.routeStepInnerTolerance,
        centerRadiusTolerance: this.props.routeStepCenterTolerance,
        courseTolerance: this.props.routeStepCourseTolerance
      },
      (trap) => {
        if (!nextStep && trap.isCenter()) {
          if (this.props.onNavigationCompleted) {
            this.props.onNavigationCompleted();
          }
          return this.setState({
            navigationMode: NavigationModes_default.IDLE,
            stepIndex: false
          });
        }
        if (trap.isLeaving()) {
          this.updateStep(this.stepIndex);
        }
      }
    );
    this.stepIndex = stepIndex + 1;
  }
  /**
   * setPosition
   * @param position Position to set
   */
  setPosition(position) {
    const { coordinate, heading } = position;
    this.traps.execute(position);
    if (this.state.navigationMode === NavigationModes_default.NAVIGATION) {
      this.updatePosition(coordinate);
      if (heading !== void 0) {
        this.updateBearing(heading);
      }
    }
    this.setState({ position });
  }
  /**
   * clearRoute
   * @void
   */
  clearRoute() {
    this.setState({ route: false, step: false, stepIndex: false });
  }
  /**
   * updateRoute
   * @param origin Origin point
   * @param destination Destination point
   * @param navigationMode Navigation mode
   * @param options Route options
   */
  updateRoute(origin = false, destination = false, navigationMode = false, options = null) {
    origin = origin || this.props.origin;
    destination = destination || this.props.destination;
    navigationMode = navigationMode || this.props.navigationMode;
    options = options || this.props.options;
    if (!origin || !destination) return;
    switch (navigationMode) {
      case NavigationModes_default.ROUTE:
        this.displayRoute(origin, destination, options);
        break;
      case NavigationModes_default.NAVIGATION:
        this.navigateRoute(origin, destination, options);
        break;
    }
  }
  /**
   * Prepares the route
   * @param origin Origin point
   * @param destination Destination point
   * @param options Route options
   * @param testForRoute Whether to test for existing route
   * @returns Promise resolving to route
   */
  prepareRoute(origin, destination, options = false, testForRoute = false) {
    if (testForRoute && this.state.route) {
      return Promise.resolve(this.state.route);
    }
    const routeOptions = Object.assign(
      {},
      { mode: this.state.travelMode },
      { mode: this.props.travelMode },
      options && typeof options === "object" ? options : {}
    );
    return this.directionsCoder.fetch(origin, destination, routeOptions).then((routes) => {
      if (routes.length) {
        const route = routes[0];
        if (!route) throw new Error("No routes found");
        if (this.props.onRouteChange) {
          this.props.onRouteChange(route);
        }
        if (this.props.onStepChange) {
          this.props.onStepChange(false);
        }
        this.setState({ route, step: false });
        return Promise.resolve(route);
      }
      return Promise.reject("No routes found");
    });
  }
  /**
   * displayRoute
   * @param origin Origin point
   * @param destination Destination point
   * @param options Route options
   * @returns Promise resolving to route
   */
  displayRoute(origin, destination, options = false) {
    return this.prepareRoute(origin, destination, options).then((route) => {
      if (!this.props.map) return Promise.resolve(route);
      const region = {
        ...route.bounds.center,
        ...this.getBoundingBoxZoomValue(
          route.bounds.boundingBox,
          this.props.directionZoomQuantifier
        )
      };
      this.props.map().animateToRegion(region, this.props.animationDuration);
      if (this.state.navigationMode !== NavigationModes_default.ROUTE) {
        this.setState({
          navigationMode: NavigationModes_default.ROUTE
        });
      }
      return Promise.resolve(route);
    }).catch((err) => {
      Logger_default.error(err);
      return Promise.reject(err);
    });
  }
  /**
   * navigateRoute
   * @param origin Origin point
   * @param destination Destination point
   * @param options Route options
   * @returns Promise resolving to route
   */
  navigateRoute(origin, destination, options = false) {
    return this.prepareRoute(origin, destination, options, true).then((route) => {
      if (!this.props.map) return Promise.resolve(route);
      const region = {
        ...route.origin.coordinate,
        ...this.getZoomValue(this.props.navigationZoomLevel)
      };
      this.props.map().animateToRegion(region, this.props.animationDuration);
      this.props.map().animateCamera(
        {
          pitch: this.props.navigationViewingAngle
        },
        {
          duration: this.props.animationDuration
        }
      );
      this.updateBearing(route.initialBearing);
      this.setState({
        navigationMode: NavigationModes_default.NAVIGATION
      });
      this.updateStep(0);
      if (this.props.onNavigationStarted) {
        this.props.onNavigationStarted();
      }
      if (this.props.simulate) {
        Logger_default.info("SIMULATING ROUTE");
        this.simulator = new Simulator(this);
        setTimeout(() => this.simulator.start(route), (this.props.animationDuration || 0) * 1.5);
      } else {
        Logger_default.info("NOT SIMULATING");
      }
      return Promise.resolve(route);
    });
  }
  /**
   * getRouteMarkers
   * @param route Route data
   * @returns Route markers as JSX
   */
  getRouteMarkers(route) {
    if (!route || route.markers.constructor !== Array) return null;
    return route.markers.map((params, index) => {
      return /* @__PURE__ */ jsx(RouterMarker, { theme: this.props.theme, ...params }, index);
    });
  }
  /**
   * getPositionMarker
   * @param position Position data
   * @param navigationMode Current navigation mode
   * @returns Position marker as JSX
   */
  getPositionMarker(position, navigationMode) {
    if (!position || !position.coordinate.latitude) return null;
    const type = navigationMode === NavigationModes_default.NAVIGATION ? "POSITION_ARROW" /* POSITION_ARROW */ : void 0;
    return /* @__PURE__ */ jsx(PositionMarker, { theme: this.props.theme, type, ...position }, "position");
  }
  /**
   * Route Polycons
   * @param route Route data
   * @returns Route polylines as JSX
   */
  getRoutePolylines(route) {
    if (!route || route.polylines.constructor !== Array) return null;
    return route.polylines.map((params, index) => {
      return params ? /* @__PURE__ */ jsx(RoutePolyline, { theme: this.props.theme, ...params }, index) : null;
    });
  }
  /**
   * getDebugShapes
   * @param route Route data
   * @returns Debug shapes as JSX
   */
  getDebugShapes(route) {
    const result = [];
    if (!route || !this.props.displayDebugMarkers) return result;
    const steps = route.steps;
    let c = 0;
    steps.forEach((step) => {
      const coordinate = step.start;
      [
        { radius: this.props.routeStepDistance, color: "blue" },
        {
          radius: this.props.routeStepDistance * this.props.routeStepInnerTolerance,
          color: "red"
        },
        {
          radius: this.props.routeStepDistance * this.props.routeStepCenterTolerance,
          color: "green"
        }
      ].forEach((d) => {
        result.push(
          /* @__PURE__ */ jsx(
            Circle,
            {
              strokeColor: d.color,
              strokeWidth: 2,
              center: step.start,
              radius: d.radius
            },
            c
          )
        );
        c++;
      });
      [{ radius: this.props.routeStepDistance, color: "blue" }].forEach((d) => {
        const bearing = step.bearing;
        const coords = toArcPolygon(
          coordinate,
          bearing - this.props.routeStepCourseTolerance,
          bearing + this.props.routeStepCourseTolerance,
          this.props.routeStepDistance
        );
        result.push(
          /* @__PURE__ */ jsx(Polyline, { strokeColor: d.color, strokeWidth: 8, coordinates: coords }, c)
        );
        c++;
      });
    });
    return result;
  }
  /**
   * @render
   * @returns {*}
   */
  render() {
    return /* @__PURE__ */ jsxs(Fragment, { children: [
      this.getRouteMarkers(this.state.route),
      this.getRoutePolylines(this.state.route),
      this.getPositionMarker(this.state.position, this.state.navigationMode),
      this.getDebugShapes(this.state.route)
    ] });
  }
};
/**
 * Default props for the component
 */
MapViewNavigation.defaultProps = {
  origin: false,
  destination: false,
  language: void 0,
  map: void 0,
  navigationMode: NavigationModes_default.IDLE,
  travelMode: TravelModes_default.DRIVING,
  maxZoom: 21,
  minZoom: 5,
  animationDuration: 750,
  navigationViewingAngle: 60,
  navigationZoomLevel: 14,
  directionZoomQuantifier: 1.5,
  onRouteChange: void 0,
  onStepChange: void 0,
  onNavigationStarted: void 0,
  onNavigationCompleted: void 0,
  routeStepDistance: 15,
  routeStepInnerTolerance: 0.75,
  routeStepCenterTolerance: 0.1,
  routeStepCourseTolerance: 30,
  // in degrees
  displayDebugMarkers: false,
  simulate: false,
  options: {}
};
var styles_default8 = (props) => StyleSheet.create({
  /**
   * @maneuverView
   */
  maneuverClose: {
    alignItems: "flex-end",
    flex: 0,
    justifyContent: "flex-end",
    width: 30
  },
  maneuverView: {
    alignItems: "center",
    backgroundColor: props.backgroundColor,
    flexDirection: "row",
    minHeight: 120,
    padding: 15
  },
  maneuverViewArrow: {
    alignItems: "center",
    flex: 0,
    justifyContent: "center",
    width: 80
  },
  maneuverViewDirection: {
    flex: 1
  }
});
var styles_default9 = (props) => StyleSheet.create({
  /**
   * @maneuverView
   */
  closeButtonText: {
    color: props.color,
    fontFamily: "Navigation",
    fontSize: props.size,
    opacity: props.opacity,
    textAlign: "center"
  }
});
var CloseButton = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {JSX.Element}
   */
  render() {
    const { size = 25, opacity = 1, color = "#000000", onPress } = this.props;
    const styles6 = styles_default9({ size, opacity, color });
    return /* @__PURE__ */ jsx(TouchableOpacity, { onPress, children: /* @__PURE__ */ jsx(Text, { style: styles6.closeButtonText, children: NavigationIcons_default.close }) });
  }
};
/**
 * defaultProps
 */
CloseButton.defaultProps = {
  size: 25,
  opacity: 1,
  color: "#000000"
};
var _ManeuverView = class _ManeuverView extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props ?? _ManeuverView.defaultProps);
  }
  /**
   * render
   * @returns {JSX.Element | null}
   */
  render() {
    const styles6 = styles_default8({
      backgroundColor: this.props.backgroundColor ?? _ManeuverView.defaultProps.backgroundColor
    });
    const step = this.props.step;
    if (!step) return null;
    const maneuver = step.maneuver;
    return /* @__PURE__ */ jsxs(TouchableOpacity, { style: styles6.maneuverView, children: [
      /* @__PURE__ */ jsx(View, { style: styles6.maneuverViewArrow, children: /* @__PURE__ */ jsx(
        ManeuverArrow,
        {
          size: this.props.arrowSize,
          color: this.props.arrowColor,
          maneuver
        }
      ) }),
      /* @__PURE__ */ jsx(View, { style: styles6.maneuverViewDirection, children: /* @__PURE__ */ jsx(
        ManeuverLabel,
        {
          ...this.props,
          instructions: step.instructions,
          fontSize: this.props.fontSize,
          fontColor: this.props.fontColor
        }
      ) }),
      !this.props.withCloseButton ? null : /* @__PURE__ */ jsx(View, { style: styles6.maneuverClose, children: /* @__PURE__ */ jsx(CloseButton, { onPress: () => this.props.onClose && this.props.onClose() }) })
    ] });
  }
};
/**
 * defaultProps
 */
_ManeuverView.defaultProps = {
  step: void 0,
  fontFamily: void 0,
  fontFamilyBold: void 0,
  backgroundColor: "#f7f7f4",
  fontSize: 20,
  arrowSize: 50,
  arrowColor: "#545455",
  withCloseButton: false,
  onClose: void 0,
  onPress: void 0
};
var ManeuverView = _ManeuverView;
var styles5 = (props) => StyleSheet.create({
  /**
   * @maneuverView
   */
  durationDistanceClose: {
    alignItems: "flex-end",
    flex: 0,
    justifyContent: "flex-end",
    width: 30
  },
  durationDistanceContent: {
    flex: 1
  },
  durationDistanceView: {
    alignItems: props.alignItems || "center",
    backgroundColor: props.backgroundColor || "#f7f7f4",
    flexDirection: props.flexDirection || "row",
    minHeight: props.minHeight || 120,
    padding: props.padding || 15
  }
});
var styles_default10 = styles5;
var DurationDistanceView = class extends Component {
  /**
   * @constructor
   * @param props
   */
  constructor(props) {
    super(props);
  }
  /**
   * render
   * @returns {XML}
   */
  render() {
    const styles6 = styles_default10(this.props);
    const step = this.props.step;
    if (!step) return null;
    return /* @__PURE__ */ jsxs(TouchableOpacity, { style: styles6.durationDistanceView, children: [
      /* @__PURE__ */ jsxs(View, { style: styles6.durationDistanceContent, children: [
        /* @__PURE__ */ jsx(Text, { children: step.distance ? step.distance.text : "" }),
        /* @__PURE__ */ jsx(Text, { children: step.duration ? step.duration.text : "" })
      ] }),
      !this.props.withCloseButton ? null : /* @__PURE__ */ jsx(View, { style: styles6.durationDistanceClose, children: /* @__PURE__ */ jsx(CloseButton, { onPress: () => this.props.onClose && this.props.onClose() }) })
    ] });
  }
};
/**
 * defaultProps
 */
DurationDistanceView.defaultProps = {
  step: void 0,
  fontFamily: void 0,
  fontFamilyBold: void 0,
  fontSize: 20,
  arrowSize: 50,
  arrowColor: "#545455",
  withCloseButton: false,
  onClose: void 0,
  onPress: void 0
};

// index.ts
var index_default = MapViewNavigation;

export { DirectionsListView, DurationDistanceLabel, DurationDistanceView, Geocoder, ManeuverArrow, ManeuverLabel, ManeuverView, NavigationModes_default as NavigationModes, NavigationIcons_default as TravelIcons, TravelModeBox, TravelModeLabel, TravelModes_default as TravelModes, index_default as default };
//# sourceMappingURL=index.mjs.map
//# sourceMappingURL=index.mjs.map